@@ -15,7 +15,6 @@ import (
 	"github.com/containers/podman/v4/pkg/rootless"
 	"github.com/containers/podman/v4/pkg/specgen"
 	"github.com/containers/podman/v4/pkg/specgenutil"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	"golang.org/x/term"
@@ -61,6 +60,7 @@ func runFlags(cmd *cobra.Command) {
 	flags := cmd.Flags()
 
 	flags.SetInterspersed(false)
+	common.DefineCreateDefaults(&cliVals)
 	common.DefineCreateFlags(cmd, &cliVals, false, false)
 	common.DefineNetFlags(cmd)
 
@@ -109,7 +109,9 @@ func init() {
 }
 
 func run(cmd *cobra.Command, args []string) error {
-	var err error
+	if err := commonFlags(cmd); err != nil {
+		return err
+	}
 
 	// TODO: Breaking change should be made fatal in next major Release
 	if cliVals.TTY && cliVals.Interactive && !term.IsTerminal(int(os.Stdin.Fd())) {
@@ -122,20 +124,16 @@ func run(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	flags := cmd.Flags()
-	cliVals.Net, err = common.NetFlagsToNetOptions(nil, *flags)
-	if err != nil {
-		return err
-	}
 	runOpts.CIDFile = cliVals.CIDFile
 	runOpts.Rm = cliVals.Rm
-	if cliVals, err = CreateInit(cmd, cliVals, false); err != nil {
+	cliVals, err := CreateInit(cmd, cliVals, false)
+	if err != nil {
 		return err
 	}
 
 	for fd := 3; fd < int(3+runOpts.PreserveFDs); fd++ {
 		if !rootless.IsFdInherited(fd) {
-			return errors.Errorf("file descriptor %d is not available - the preserve-fds option requires that file descriptors must be passed", fd)
+			return fmt.Errorf("file descriptor %d is not available - the preserve-fds option requires that file descriptors must be passed", fd)
 		}
 	}
 
@@ -143,7 +141,7 @@ func run(cmd *cobra.Command, args []string) error {
 	rawImageName := ""
 	if !cliVals.RootFS {
 		rawImageName = args[0]
-		name, err := PullImage(args[0], cliVals)
+		name, err := PullImage(args[0], &cliVals)
 		if err != nil {
 			return err
 		}
@@ -166,7 +164,7 @@ func run(cmd *cobra.Command, args []string) error {
 	// If attach is set, clear stdin/stdout/stderr and only attach requested
 	if cmd.Flag("attach").Changed {
 		if passthrough {
-			return errors.Wrapf(define.ErrInvalidArg, "cannot specify --attach with --log-driver=passthrough")
+			return fmt.Errorf("cannot specify --attach with --log-driver=passthrough: %w", define.ErrInvalidArg)
 		}
 		runOpts.OutputStream = nil
 		runOpts.ErrorStream = nil
@@ -183,7 +181,7 @@ func run(cmd *cobra.Command, args []string) error {
 			case "stdin":
 				runOpts.InputStream = os.Stdin
 			default:
-				return errors.Wrapf(define.ErrInvalidArg, "invalid stream %q for --attach - must be one of stdin, stdout, or stderr", stream)
+				return fmt.Errorf("invalid stream %q for --attach - must be one of stdin, stdout, or stderr: %w", stream, define.ErrInvalidArg)
 			}
 		}
 	}
@@ -194,6 +192,9 @@ func run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 	s.RawImageName = rawImageName
+	s.ImageOS = cliVals.OS
+	s.ImageArch = cliVals.Arch
+	s.ImageVariant = cliVals.Variant
 	s.Passwd = &runOpts.Passwd
 	runOpts.Spec = s
 
