@@ -142,7 +142,10 @@ exec_container() {
     # Line-separated arguments which include shell-escaped special characters
     declare -a envargs
     while read -r var_val; do
-        envargs+=("-e $var_val")
+        # Pass "-e VAR" on the command line, not "-e VAR=value". Podman can
+        # do a much better job of transmitting the value than we can,
+        # especially when value includes spaces.
+        envargs+=("-e" "$(awk -F= '{print $1}' <<<$var_val)")
     done <<<"$(passthrough_envars)"
 
     # VM Images and Container images are built using (nearly) identical operations.
@@ -233,6 +236,16 @@ function _run_build() {
     make clean
     make vendor
     make podman-release  # includes podman, podman-remote, and docs
+
+    # Last-minute confirmation that we're testing the desired runtime.
+    # This Can't Possibly Fail™ in regular CI; only when updating VMs.
+    # $CI_DESIRED_RUNTIME must be defined in .cirrus.yml.
+    req_env_vars CI_DESIRED_RUNTIME
+    runtime=$(bin/podman info --format '{{.Host.OCIRuntime.Name}}')
+    # shellcheck disable=SC2154
+    if [[ "$runtime" != "$CI_DESIRED_RUNTIME" ]]; then
+        die "Built podman is using '$runtime'; this CI environment requires $CI_DESIRED_RUNTIME"
+    fi
 }
 
 function _run_altbuild() {
@@ -246,20 +259,28 @@ function _run_altbuild() {
     # shellcheck disable=SC2154
     msg "Performing alternate build: $ALT_NAME"
     msg "************************************************************"
+    set -x
     cd $GOSRC
     case "$ALT_NAME" in
         *Each*)
             git fetch origin
-            # The check-size script, introduced 2022-03-22 in #13518,
+            # The make-and-check-size script, introduced 2022-03-22 in #13518,
             # runs 'make' (the original purpose of this check) against
             # each commit, then checks image sizes to make sure that
             # none have grown beyond a given limit. That of course
-            # requires a baseline, which is why we use '^' to start
-            # with the *parent* commit of this PR, not the first commit.
+            # requires a baseline, so our first step is to build the
+            # branch point of the PR.
+            local context_dir savedhead pr_base
             context_dir=$(mktemp -d --tmpdir make-size-check.XXXXXXX)
-            make build-all-new-commits \
-                 GIT_BASE_BRANCH=origin/"${DEST_BRANCH}^" \
-                 MAKE="hack/make-and-check-size $context_dir"
+            savedhead=$(git rev-parse HEAD)
+            # Push to PR base. First run of the script will write size files
+            pr_base=$(git merge-base origin/$DEST_BRANCH HEAD)
+            git checkout $pr_base
+            hack/make-and-check-size $context_dir
+            # pop back to PR, and run incremental makes. Subsequent script
+            # invocations will compare against original size.
+            git checkout $savedhead
+            git rebase $pr_base -x "hack/make-and-check-size $context_dir"
             rm -rf $context_dir
             ;;
         *Windows*)
@@ -295,10 +316,6 @@ function _run_altbuild() {
 }
 
 function _run_release() {
-    # TODO: These tests should come from code external to the podman repo.
-    # to allow test-changes (and re-runs) in the case of a correctable test
-    # flaw or flake at release tag-push time.  For now, the test is here
-    # given its simplicity.
     msg "podman info:"
     bin/podman info
 
@@ -308,10 +325,18 @@ function _run_release() {
     if [[ -n "$dev" ]]; then
         die "Releases must never contain '-dev' in output of 'podman info' ($dev)"
     fi
+
+    commit=$(bin/podman info --format='{{.Version.GitCommit}}' | tr -d '[:space:]')
+    if [[ -z "$commit" ]]; then
+        die "Releases must contain a non-empty Version.GitCommit in 'podman info'"
+    fi
     msg "All OK"
 }
 
 
+# ***WARNING*** ***WARNING*** ***WARNING*** ***WARNING***
+#    Please see gitlab comment in setup_environment.sh
+# ***WARNING*** ***WARNING*** ***WARNING*** ***WARNING***
 function _run_gitlab() {
     rootless_uid=$(id -u)
     systemctl enable --now --user podman.socket
@@ -367,6 +392,11 @@ dotest() {
         |& logformatter
 }
 
+_run_machine() {
+    # N/B: Can't use _bail_if_test_can_be_skipped here b/c content isn't under test/
+    make localmachine |& logformatter
+}
+
 # Optimization: will exit if the only PR diffs are under docs/ or tests/
 # with the exception of any given arguments. E.g., don't run e2e or upgrade
 # or bud tests if the only PR changes are in test/system.
@@ -375,7 +405,7 @@ function _bail_if_test_can_be_skipped() {
 
     # Cirrus sets these for PRs but not branches or cron. In cron and branches,
     #we never want to skip.
-    for v in CIRRUS_CHANGE_IN_REPO CIRRUS_PR; do
+    for v in CIRRUS_CHANGE_IN_REPO CIRRUS_PR DEST_BRANCH; do
         if [[ -z "${!v}" ]]; then
             msg "[ _cannot do selective skip: \$$v is undefined ]"
             return 0
