@@ -3,6 +3,7 @@ package compat
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -16,6 +17,7 @@ import (
 	"github.com/containers/buildah"
 	buildahDefine "github.com/containers/buildah/define"
 	"github.com/containers/buildah/pkg/parse"
+	"github.com/containers/image/v5/docker/reference"
 	"github.com/containers/image/v5/types"
 	"github.com/containers/podman/v4/libpod"
 	"github.com/containers/podman/v4/pkg/api/handlers/utils"
@@ -27,7 +29,6 @@ import (
 	"github.com/docker/docker/pkg/jsonmessage"
 	"github.com/gorilla/schema"
 	"github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 )
 
@@ -65,77 +66,84 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 		err := os.RemoveAll(filepath.Dir(contextDirectory))
 		if err != nil {
-			logrus.Warn(errors.Wrapf(err, "failed to remove build scratch directory %q", filepath.Dir(contextDirectory)))
+			logrus.Warn(fmt.Errorf("failed to remove build scratch directory %q: %w", filepath.Dir(contextDirectory), err))
 		}
 	}()
 
 	query := struct {
-		AddHosts               string   `schema:"extrahosts"`
-		AdditionalCapabilities string   `schema:"addcaps"`
-		AllPlatforms           bool     `schema:"allplatforms"`
-		Annotations            string   `schema:"annotations"`
-		AppArmor               string   `schema:"apparmor"`
-		BuildArgs              string   `schema:"buildargs"`
-		CacheFrom              string   `schema:"cachefrom"`
-		CgroupParent           string   `schema:"cgroupparent"` // nolint
-		Compression            uint64   `schema:"compression"`
-		ConfigureNetwork       string   `schema:"networkmode"`
-		CpuPeriod              uint64   `schema:"cpuperiod"`  // nolint
-		CpuQuota               int64    `schema:"cpuquota"`   // nolint
-		CpuSetCpus             string   `schema:"cpusetcpus"` // nolint
-		CpuSetMems             string   `schema:"cpusetmems"` // nolint
-		CpuShares              uint64   `schema:"cpushares"`  // nolint
-		DNSOptions             string   `schema:"dnsoptions"`
-		DNSSearch              string   `schema:"dnssearch"`
-		DNSServers             string   `schema:"dnsservers"`
-		Devices                string   `schema:"devices"`
-		Dockerfile             string   `schema:"dockerfile"`
-		DropCapabilities       string   `schema:"dropcaps"`
-		Envs                   []string `schema:"setenv"`
-		Excludes               string   `schema:"excludes"`
-		ForceRm                bool     `schema:"forcerm"`
-		From                   string   `schema:"from"`
-		HTTPProxy              bool     `schema:"httpproxy"`
-		IdentityLabel          bool     `schema:"identitylabel"`
-		Ignore                 bool     `schema:"ignore"`
-		Isolation              string   `schema:"isolation"`
-		Jobs                   int      `schema:"jobs"` // nolint
-		LabelOpts              string   `schema:"labelopts"`
-		Labels                 string   `schema:"labels"`
-		Layers                 bool     `schema:"layers"`
-		LogRusage              bool     `schema:"rusage"`
-		Manifest               string   `schema:"manifest"`
-		MemSwap                int64    `schema:"memswap"`
-		Memory                 int64    `schema:"memory"`
-		NamespaceOptions       string   `schema:"nsoptions"`
-		NoCache                bool     `schema:"nocache"`
-		OSFeatures             []string `schema:"osfeature"`
-		OSVersion              string   `schema:"osversion"`
-		OutputFormat           string   `schema:"outputformat"`
-		Platform               []string `schema:"platform"`
-		Pull                   bool     `schema:"pull"`
-		PullPolicy             string   `schema:"pullpolicy"`
-		Quiet                  bool     `schema:"q"`
-		Registry               string   `schema:"registry"`
-		Rm                     bool     `schema:"rm"`
-		RusageLogFile          string   `schema:"rusagelogfile"`
-		Seccomp                string   `schema:"seccomp"`
-		Secrets                string   `schema:"secrets"`
-		SecurityOpt            string   `schema:"securityopt"`
-		ShmSize                int      `schema:"shmsize"`
-		Squash                 bool     `schema:"squash"`
-		TLSVerify              bool     `schema:"tlsVerify"`
-		Tags                   []string `schema:"t"`
-		Target                 string   `schema:"target"`
-		Timestamp              int64    `schema:"timestamp"`
-		Ulimits                string   `schema:"ulimits"`
-		UnsetEnvs              []string `schema:"unsetenv"`
+		AddHosts                string   `schema:"extrahosts"`
+		AdditionalCapabilities  string   `schema:"addcaps"`
+		AdditionalBuildContexts string   `schema:"additionalbuildcontexts"`
+		AllPlatforms            bool     `schema:"allplatforms"`
+		Annotations             string   `schema:"annotations"`
+		AppArmor                string   `schema:"apparmor"`
+		BuildArgs               string   `schema:"buildargs"`
+		CacheFrom               string   `schema:"cachefrom"`
+		CacheTo                 string   `schema:"cacheto"`
+		CacheTTL                string   `schema:"cachettl"`
+		CgroupParent            string   `schema:"cgroupparent"`
+		Compression             uint64   `schema:"compression"`
+		ConfigureNetwork        string   `schema:"networkmode"`
+		CPPFlags                string   `schema:"cppflags"`
+		CpuPeriod               uint64   `schema:"cpuperiod"`  //nolint:revive,stylecheck
+		CpuQuota                int64    `schema:"cpuquota"`   //nolint:revive,stylecheck
+		CpuSetCpus              string   `schema:"cpusetcpus"` //nolint:revive,stylecheck
+		CpuSetMems              string   `schema:"cpusetmems"` //nolint:revive,stylecheck
+		CpuShares               uint64   `schema:"cpushares"`  //nolint:revive,stylecheck
+		DNSOptions              string   `schema:"dnsoptions"`
+		DNSSearch               string   `schema:"dnssearch"`
+		DNSServers              string   `schema:"dnsservers"`
+		Devices                 string   `schema:"devices"`
+		Dockerfile              string   `schema:"dockerfile"`
+		DropCapabilities        string   `schema:"dropcaps"`
+		Envs                    []string `schema:"setenv"`
+		Excludes                string   `schema:"excludes"`
+		ForceRm                 bool     `schema:"forcerm"`
+		From                    string   `schema:"from"`
+		HTTPProxy               bool     `schema:"httpproxy"`
+		IdentityLabel           bool     `schema:"identitylabel"`
+		Ignore                  bool     `schema:"ignore"`
+		Isolation               string   `schema:"isolation"`
+		Jobs                    int      `schema:"jobs"`
+		LabelOpts               string   `schema:"labelopts"`
+		Labels                  string   `schema:"labels"`
+		Layers                  bool     `schema:"layers"`
+		LogRusage               bool     `schema:"rusage"`
+		Manifest                string   `schema:"manifest"`
+		MemSwap                 int64    `schema:"memswap"`
+		Memory                  int64    `schema:"memory"`
+		NamespaceOptions        string   `schema:"nsoptions"`
+		NoCache                 bool     `schema:"nocache"`
+		OmitHistory             bool     `schema:"omithistory"`
+		OSFeatures              []string `schema:"osfeature"`
+		OSVersion               string   `schema:"osversion"`
+		OutputFormat            string   `schema:"outputformat"`
+		Platform                []string `schema:"platform"`
+		Pull                    bool     `schema:"pull"`
+		PullPolicy              string   `schema:"pullpolicy"`
+		Quiet                   bool     `schema:"q"`
+		Registry                string   `schema:"registry"`
+		Rm                      bool     `schema:"rm"`
+		RusageLogFile           string   `schema:"rusagelogfile"`
+		Remote                  string   `schema:"remote"`
+		Seccomp                 string   `schema:"seccomp"`
+		Secrets                 string   `schema:"secrets"`
+		SecurityOpt             string   `schema:"securityopt"`
+		ShmSize                 int      `schema:"shmsize"`
+		Squash                  bool     `schema:"squash"`
+		TLSVerify               bool     `schema:"tlsVerify"`
+		Tags                    []string `schema:"t"`
+		Target                  string   `schema:"target"`
+		Timestamp               int64    `schema:"timestamp"`
+		Ulimits                 string   `schema:"ulimits"`
+		UnsetEnvs               []string `schema:"unsetenv"`
 	}{
 		Dockerfile:    "Dockerfile",
 		IdentityLabel: true,
 		Registry:      "docker.io",
 		Rm:            true,
 		ShmSize:       64 * 1024 * 1024,
+		TLSVerify:     true,
 	}
 
 	decoder := r.Context().Value(api.DecoderKey).(*schema.Decoder)
@@ -169,14 +177,50 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 	// convert addcaps formats
 	containerFiles := []string{}
-	if _, found := r.URL.Query()["dockerfile"]; found {
-		var m = []string{}
-		if err := json.Unmarshal([]byte(query.Dockerfile), &m); err != nil {
-			// it's not json, assume just a string
-			m = []string{filepath.Join(contextDirectory, query.Dockerfile)}
+	// Tells if query parameter `dockerfile` is set or not.
+	dockerFileSet := false
+	if utils.IsLibpodRequest(r) && query.Remote != "" {
+		// The context directory could be a URL.  Try to handle that.
+		anchorDir, err := ioutil.TempDir(parse.GetTempDir(), "libpod_builder")
+		if err != nil {
+			utils.InternalServerError(w, err)
+		}
+		tempDir, subDir, err := buildahDefine.TempDirForURL(anchorDir, "buildah", query.Remote)
+		if err != nil {
+			utils.InternalServerError(w, err)
+		}
+		if tempDir != "" {
+			// We had to download it to a temporary directory.
+			// Delete it later.
+			defer func() {
+				if err = os.RemoveAll(tempDir); err != nil {
+					// We are deleting this on server so log on server end
+					// client does not have to worry about server cleanup.
+					logrus.Errorf("Cannot delete downloaded temp dir %q: %s", tempDir, err)
+				}
+			}()
+			contextDirectory = filepath.Join(tempDir, subDir)
+		} else {
+			// Nope, it was local.  Use it as is.
+			absDir, err := filepath.Abs(query.Remote)
+			if err != nil {
+				utils.BadRequest(w, "remote", query.Remote, err)
+			}
+			contextDirectory = absDir
 		}
-		containerFiles = m
 	} else {
+		if _, found := r.URL.Query()["dockerfile"]; found {
+			var m = []string{}
+			if err := json.Unmarshal([]byte(query.Dockerfile), &m); err != nil {
+				// it's not json, assume just a string
+				m = []string{filepath.Join(contextDirectory, query.Dockerfile)}
+			}
+			containerFiles = m
+			dockerFileSet = true
+		}
+	}
+
+	if !dockerFileSet {
 		containerFiles = []string{filepath.Join(contextDirectory, "Dockerfile")}
 		if utils.IsLibpodRequest(r) {
 			containerFiles = []string{filepath.Join(contextDirectory, "Containerfile")}
@@ -298,7 +342,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	if len(tags) > 0 {
 		possiblyNormalizedName, err := utils.NormalizeToDockerHub(r, tags[0])
 		if err != nil {
-			utils.Error(w, http.StatusInternalServerError, errors.Wrap(err, "error normalizing image"))
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error normalizing image: %w", err))
 			return
 		}
 		output = possiblyNormalizedName
@@ -310,7 +354,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		var err error
 		isolation, err = parseLibPodIsolation(query.Isolation)
 		if err != nil {
-			utils.Error(w, http.StatusInternalServerError, errors.Wrap(err, "failed to parse isolation"))
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("failed to parse isolation: %w", err))
 			return
 		}
 
@@ -327,16 +371,49 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			}
 		}
 	}
-	var additionalTags []string // nolint
+	var additionalTags []string
 	for i := 1; i < len(tags); i++ {
 		possiblyNormalizedTag, err := utils.NormalizeToDockerHub(r, tags[i])
 		if err != nil {
-			utils.Error(w, http.StatusInternalServerError, errors.Wrap(err, "error normalizing image"))
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error normalizing image: %w", err))
 			return
 		}
 		additionalTags = append(additionalTags, possiblyNormalizedTag)
 	}
 
+	var additionalBuildContexts = map[string]*buildahDefine.AdditionalBuildContext{}
+	if _, found := r.URL.Query()["additionalbuildcontexts"]; found {
+		if err := json.Unmarshal([]byte(query.AdditionalBuildContexts), &additionalBuildContexts); err != nil {
+			utils.BadRequest(w, "additionalbuildcontexts", query.AdditionalBuildContexts, err)
+			return
+		}
+	}
+
+	var cacheFrom reference.Named
+	if _, found := r.URL.Query()["cachefrom"]; found {
+		cacheFrom, err = parse.RepoNameToNamedReference(query.CacheFrom)
+		if err != nil {
+			utils.BadRequest(w, "cacheFrom", query.CacheFrom, err)
+			return
+		}
+	}
+	var cacheTo reference.Named
+	if _, found := r.URL.Query()["cacheto"]; found {
+		cacheTo, err = parse.RepoNameToNamedReference(query.CacheTo)
+		if err != nil {
+			utils.BadRequest(w, "cacheto", query.CacheTo, err)
+			return
+		}
+	}
+	var cacheTTL time.Duration
+	if _, found := r.URL.Query()["cachettl"]; found {
+		cacheTTL, err = time.ParseDuration(query.CacheTTL)
+		if err != nil {
+			utils.BadRequest(w, "cachettl", query.CacheTTL, err)
+			return
+		}
+	}
+
 	var buildArgs = map[string]string{}
 	if _, found := r.URL.Query()["buildargs"]; found {
 		if err := json.Unmarshal([]byte(query.BuildArgs), &buildArgs); err != nil {
@@ -362,6 +439,15 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 	}
 
+	// convert cppflags formats
+	var cppflags = []string{}
+	if _, found := r.URL.Query()["cppflags"]; found {
+		if err := json.Unmarshal([]byte(query.CPPFlags), &cppflags); err != nil {
+			utils.BadRequest(w, "cppflags", query.CPPFlags, err)
+			return
+		}
+	}
+
 	// convert nsoptions formats
 	nsoptions := buildah.NamespaceOptions{}
 	if _, found := r.URL.Query()["nsoptions"]; found {
@@ -430,7 +516,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 				}
 				con := strings.SplitN(opt, "=", 2)
 				if len(con) != 2 {
-					utils.BadRequest(w, "securityopt", query.SecurityOpt, errors.Errorf("Invalid --security-opt name=value pair: %q", opt))
+					utils.BadRequest(w, "securityopt", query.SecurityOpt, fmt.Errorf("invalid --security-opt name=value pair: %q", opt))
 					return
 				}
 
@@ -442,7 +528,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 				case "seccomp":
 					seccomp = con[1]
 				default:
-					utils.BadRequest(w, "securityopt", query.SecurityOpt, errors.Errorf("Invalid --security-opt 2: %q", opt))
+					utils.BadRequest(w, "securityopt", query.SecurityOpt, fmt.Errorf("invalid --security-opt 2: %q", opt))
 					return
 				}
 			}
@@ -483,7 +569,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	if fromImage != "" {
 		possiblyNormalizedName, err := utils.NormalizeToDockerHub(r, fromImage)
 		if err != nil {
-			utils.Error(w, http.StatusInternalServerError, errors.Wrap(err, "error normalizing image"))
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error normalizing image: %w", err))
 			return
 		}
 		fromImage = possiblyNormalizedName
@@ -515,11 +601,16 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 	runtime := r.Context().Value(api.RuntimeKey).(*libpod.Runtime)
 	buildOptions := buildahDefine.BuildOptions{
-		AddCapabilities: addCaps,
-		AdditionalTags:  additionalTags,
-		Annotations:     annotations,
-		Args:            buildArgs,
-		AllPlatforms:    query.AllPlatforms,
+		AddCapabilities:         addCaps,
+		AdditionalBuildContexts: additionalBuildContexts,
+		AdditionalTags:          additionalTags,
+		Annotations:             annotations,
+		CPPFlags:                cppflags,
+		CacheFrom:               cacheFrom,
+		CacheTo:                 cacheTo,
+		CacheTTL:                cacheTTL,
+		Args:                    buildArgs,
+		AllPlatforms:            query.AllPlatforms,
 		CommonBuildOpts: &buildah.CommonBuildOptions{
 			AddHost:            addhosts,
 			ApparmorProfile:    apparmor,
@@ -537,6 +628,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			LabelOpts:          labelOpts,
 			Memory:             query.Memory,
 			MemorySwap:         query.MemSwap,
+			OmitHistory:        query.OmitHistory,
 			SeccompProfilePath: seccomp,
 			ShmSize:            strconv.Itoa(query.ShmSize),
 			Ulimit:             ulimits,
@@ -568,7 +660,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		Output:                         output,
 		OutputFormat:                   format,
 		PullPolicy:                     pullPolicy,
-		PullPushRetryDelay:             time.Duration(2 * time.Second),
+		PullPushRetryDelay:             2 * time.Second,
 		Quiet:                          query.Quiet,
 		Registry:                       registry,
 		RemoveIntermediateCtrs:         query.Rm,
@@ -637,15 +729,17 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 	enc := json.NewEncoder(body)
 	enc.SetEscapeHTML(true)
+	var stepErrors []string
 
 	for {
-		m := struct {
+		type BuildResponse struct {
 			Stream string                 `json:"stream,omitempty"`
 			Error  *jsonmessage.JSONError `json:"errorDetail,omitempty"`
 			// NOTE: `error` is being deprecated check https://github.com/moby/moby/blob/master/pkg/jsonmessage/jsonmessage.go#L148
 			ErrorMessage string          `json:"error,omitempty"` // deprecate this slowly
 			Aux          json.RawMessage `json:"aux,omitempty"`
-		}{}
+		}
+		m := BuildResponse{}
 
 		select {
 		case e := <-stdout.Chan():
@@ -661,12 +755,27 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			}
 			flush()
 		case e := <-auxout.Chan():
-			m.Stream = string(e)
-			if err := enc.Encode(m); err != nil {
-				stderr.Write([]byte(err.Error()))
+			if !query.Quiet {
+				m.Stream = string(e)
+				if err := enc.Encode(m); err != nil {
+					stderr.Write([]byte(err.Error()))
+				}
+				flush()
+			} else {
+				stepErrors = append(stepErrors, string(e))
 			}
-			flush()
 		case e := <-stderr.Chan():
+			// Docker-API Compat parity : Build failed so
+			// output all step errors irrespective of quiet
+			// flag.
+			for _, stepError := range stepErrors {
+				t := BuildResponse{}
+				t.Stream = stepError
+				if err := enc.Encode(t); err != nil {
+					stderr.Write([]byte(err.Error()))
+				}
+				flush()
+			}
 			m.ErrorMessage = string(e)
 			m.Error = &jsonmessage.JSONError{
 				Message: m.ErrorMessage,
@@ -724,7 +833,7 @@ func parseNetworkConfigurationPolicy(network string) buildah.NetworkConfiguratio
 	}
 }
 
-func parseLibPodIsolation(isolation string) (buildah.Isolation, error) { // nolint
+func parseLibPodIsolation(isolation string) (buildah.Isolation, error) {
 	if val, err := strconv.Atoi(isolation); err == nil {
 		return buildah.Isolation(val), nil
 	}
@@ -739,7 +848,7 @@ func extractTarFile(r *http.Request) (string, error) {
 	}
 
 	path := filepath.Join(anchorDir, "tarBall")
-	tarBall, err := os.OpenFile(path, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0600)
+	tarBall, err := os.OpenFile(path, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0o600)
 	if err != nil {
 		return "", err
 	}
@@ -753,7 +862,7 @@ func extractTarFile(r *http.Request) (string, error) {
 	}
 
 	buildDir := filepath.Join(anchorDir, "build")
-	err = os.Mkdir(buildDir, 0700)
+	err = os.Mkdir(buildDir, 0o700)
 	if err != nil {
 		return "", err
 	}
