@@ -5,7 +5,6 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
-	"strings"
 
 	. "github.com/containers/podman/v4/test/utils"
 	. "github.com/onsi/ginkgo"
@@ -53,23 +52,29 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 	})
 
-	It("podman pull from docker a not existing image", func() {
-		session := podmanTest.Podman([]string{"pull", "ibetthisdoesntexistthere:foo"})
+	It("podman pull bogus image", func() {
+		session := podmanTest.Podman([]string{"pull", "quay.io/ibetthis/doesntexistthere:foo"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).To(ExitWithError())
 	})
 
-	It("podman pull from docker with tag", func() {
-		session := podmanTest.Podman([]string{"pull", "quay.io/libpod/testdigest_v2s2:20200210"})
+	It("podman pull with tag --quiet", func() {
+		session := podmanTest.Podman([]string{"pull", "-q", "quay.io/libpod/testdigest_v2s2:20200210"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
+		quietOutput := session.OutputToString()
+
+		session = podmanTest.Podman([]string{"inspect", "testdigest_v2s2:20200210", "--format", "{{.ID}}"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).To(Equal(quietOutput))
 
 		session = podmanTest.Podman([]string{"rmi", "testdigest_v2s2:20200210"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 	})
 
-	It("podman pull from docker without tag", func() {
+	It("podman pull without tag", func() {
 		session := podmanTest.Podman([]string{"pull", "quay.io/libpod/testdigest_v2s2"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
@@ -79,26 +84,6 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 	})
 
-	It("podman pull from alternate registry with tag", func() {
-		session := podmanTest.Podman([]string{"pull", cirros})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-
-		session = podmanTest.Podman([]string{"rmi", cirros})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-	})
-
-	It("podman pull from alternate registry without tag", func() {
-		session := podmanTest.Podman([]string{"pull", "quay.io/libpod/cirros"})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-
-		session = podmanTest.Podman([]string{"rmi", "quay.io/libpod/cirros"})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-	})
-
 	It("podman pull by digest", func() {
 		session := podmanTest.Podman([]string{"pull", "quay.io/libpod/testdigest_v2s2@sha256:755f4d90b3716e2bf57060d249e2cd61c9ac089b1233465c5c2cb2d7ee550fdb"})
 		session.WaitWithDefaultTimeout()
@@ -114,6 +99,32 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 	})
 
+	It("podman pull check all tags", func() {
+		session := podmanTest.Podman([]string{"pull", "--all-tags", "quay.io/libpod/testdigest_v2s2"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		session = podmanTest.Podman([]string{"images"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(len(session.OutputToStringArray())).To(BeNumerically(">=", 2), "Expected at least two images")
+
+		session = podmanTest.Podman([]string{"pull", "-a", "quay.io/libpod/testdigest_v2s2"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		session = podmanTest.Podman([]string{"images"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(len(session.OutputToStringArray())).To(BeNumerically(">=", 2), "Expected at least two images")
+	})
+
+	It("podman pull from docker with nonexistent --authfile", func() {
+		session := podmanTest.Podman([]string{"pull", "--authfile", "/tmp/nonexistent", ALPINE})
+		session.WaitWithDefaultTimeout()
+		Expect(session).To(ExitWithError())
+	})
+
 	It("podman pull by digest (image list)", func() {
 		session := podmanTest.Podman([]string{"pull", "--arch=arm64", ALPINELISTDIGEST})
 		session.WaitWithDefaultTimeout()
@@ -251,16 +262,10 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 	})
 
-	It("podman pull bogus image", func() {
-		session := podmanTest.Podman([]string{"pull", "umohnani/get-started"})
-		session.WaitWithDefaultTimeout()
-		Expect(session).To(ExitWithError())
-	})
-
 	It("podman pull from docker-archive", func() {
 		SkipIfRemote("podman-remote does not support pulling from docker-archive")
 
-		podmanTest.AddImageToRWStore(cirros)
+		podmanTest.AddImageToRWStore(CIRROS_IMAGE)
 		tarfn := filepath.Join(podmanTest.TempDir, "cirros.tar")
 		session := podmanTest.Podman([]string{"save", "-o", tarfn, "cirros"})
 		session.WaitWithDefaultTimeout()
@@ -323,7 +328,7 @@ var _ = Describe("Podman pull", func() {
 	It("podman pull from oci-archive", func() {
 		SkipIfRemote("podman-remote does not support pulling from oci-archive")
 
-		podmanTest.AddImageToRWStore(cirros)
+		podmanTest.AddImageToRWStore(CIRROS_IMAGE)
 		tarfn := filepath.Join(podmanTest.TempDir, "oci-cirrus.tar")
 		session := podmanTest.Podman([]string{"save", "--format", "oci-archive", "-o", tarfn, "cirros"})
 		session.WaitWithDefaultTimeout()
@@ -343,9 +348,10 @@ var _ = Describe("Podman pull", func() {
 	It("podman pull from local directory", func() {
 		SkipIfRemote("podman-remote does not support pulling from local directory")
 
-		podmanTest.AddImageToRWStore(cirros)
+		podmanTest.AddImageToRWStore(CIRROS_IMAGE)
 		dirpath := filepath.Join(podmanTest.TempDir, "cirros")
-		os.MkdirAll(dirpath, os.ModePerm)
+		err = os.MkdirAll(dirpath, os.ModePerm)
+		Expect(err).ToNot(HaveOccurred())
 		imgPath := fmt.Sprintf("dir:%s", dirpath)
 
 		session := podmanTest.Podman([]string{"push", "cirros", imgPath})
@@ -366,9 +372,10 @@ var _ = Describe("Podman pull", func() {
 	It("podman pull from local OCI directory", func() {
 		SkipIfRemote("podman-remote does not support pulling from OCI directory")
 
-		podmanTest.AddImageToRWStore(cirros)
+		podmanTest.AddImageToRWStore(CIRROS_IMAGE)
 		dirpath := filepath.Join(podmanTest.TempDir, "cirros")
-		os.MkdirAll(dirpath, os.ModePerm)
+		err = os.MkdirAll(dirpath, os.ModePerm)
+		Expect(err).ToNot(HaveOccurred())
 		imgPath := fmt.Sprintf("oci:%s", dirpath)
 
 		session := podmanTest.Podman([]string{"push", "cirros", imgPath})
@@ -386,41 +393,6 @@ var _ = Describe("Podman pull", func() {
 		Expect(session.LineInOutputContainsTag(filepath.Join("localhost", dirpath), "latest")).To(BeTrue())
 	})
 
-	It("podman pull check quiet", func() {
-		podmanTest.RestoreArtifact(ALPINE)
-		setup := podmanTest.Podman([]string{"images", ALPINE, "-q", "--no-trunc"})
-		setup.WaitWithDefaultTimeout()
-		Expect(setup).Should(Exit(0))
-		shortImageID := strings.Split(setup.OutputToString(), ":")[1]
-
-		rmi := podmanTest.Podman([]string{"rmi", ALPINE})
-		rmi.WaitWithDefaultTimeout()
-		Expect(rmi).Should(Exit(0))
-
-		pull := podmanTest.Podman([]string{"pull", "-q", ALPINE})
-		pull.WaitWithDefaultTimeout()
-		Expect(pull).Should(Exit(0))
-
-		Expect(pull.OutputToString()).To(ContainSubstring(shortImageID))
-	})
-
-	It("podman pull check all tags", func() {
-		session := podmanTest.Podman([]string{"pull", "--all-tags", "k8s.gcr.io/pause"})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-
-		session = podmanTest.Podman([]string{"images"})
-		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(BeNumerically(">", 4))
-	})
-
-	It("podman pull from docker with nonexistent --authfile", func() {
-		session := podmanTest.Podman([]string{"pull", "--authfile", "/tmp/nonexistent", ALPINE})
-		session.WaitWithDefaultTimeout()
-		Expect(session).To(ExitWithError())
-	})
-
 	It("podman pull + inspect from unqualified-search registry", func() {
 		// Regression test for #6381:
 		// Make sure that `pull shortname` and `inspect shortname`
@@ -428,8 +400,10 @@ var _ = Describe("Podman pull", func() {
 
 		// We already tested pulling, so we can save some energy and
 		// just restore local artifacts and tag them.
-		podmanTest.RestoreArtifact(ALPINE)
-		podmanTest.RestoreArtifact(BB)
+		err := podmanTest.RestoreArtifact(ALPINE)
+		Expect(err).ToNot(HaveOccurred())
+		err = podmanTest.RestoreArtifact(BB)
+		Expect(err).ToNot(HaveOccurred())
 
 		// What we want is at least two images which have the same name
 		// and are prefixed with two different unqualified-search
@@ -514,7 +488,7 @@ var _ = Describe("Podman pull", func() {
 			Expect(data).To(HaveLen(1))
 			Expect(data[0].RepoTags).To(HaveLen(1))
 			Expect(data[0].RepoTags[0]).To(Equal(t.tag1))
-			Expect(data[0].ID).To(Equal(image1))
+			Expect(data[0]).To(HaveField("ID", image1))
 		}
 	})
 
@@ -541,8 +515,8 @@ var _ = Describe("Podman pull", func() {
 
 		data := setup.InspectImageJSON() // returns []inspect.ImageData
 		Expect(data).To(HaveLen(1))
-		Expect(data[0].Os).To(Equal(runtime.GOOS))
-		Expect(data[0].Architecture).To(Equal("arm64"))
+		Expect(data[0]).To(HaveField("Os", runtime.GOOS))
+		Expect(data[0]).To(HaveField("Architecture", "arm64"))
 	})
 
 	It("podman pull --arch", func() {
@@ -568,7 +542,7 @@ var _ = Describe("Podman pull", func() {
 
 		data := setup.InspectImageJSON() // returns []inspect.ImageData
 		Expect(data).To(HaveLen(1))
-		Expect(data[0].Os).To(Equal(runtime.GOOS))
-		Expect(data[0].Architecture).To(Equal("arm64"))
+		Expect(data[0]).To(HaveField("Os", runtime.GOOS))
+		Expect(data[0]).To(HaveField("Architecture", "arm64"))
 	})
 })
